/*************************************************************************
* (C) 2010 AITNET ltd - Sofia/Bulgaria - <misho@aitbg.com>
*  by Michael Pounov <misho@openbsd-bg.org>
*
* $Author: misho $
* $Id: builtin.c,v 1.23 2015/07/22 20:01:46 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2015
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#include "global.h"


/* builtin RPC server functions */

static int
rpcServerClients(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;
	rpc_cli_t *c;
	register int i;
	int len;
	const char *str = NULL;
	char *val;
	ait_val_t v;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	len = array_Size(srv->srv_clients) * STRSIZ;
	if (!(val = e_malloc(len))) {
		LOGERR;
		return -1;
	} else
		memset(val, 0, len);

	for (i = 0; i < array_Size(srv->srv_clients); i++) {
		c = array(srv->srv_clients, i, rpc_cli_t*);
		if (!c)
			continue;

		str = e_n2addr(&c->cli_sa, &v);
		if (str)
			strlcat(val, (char*) str, len);
		else
			strlcat(val, "0.0.0.0", len);
		strlcat(val, " ", len);
		AIT_FREE_VAL(&v);
	}

	/* return values */
	AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), val);
	e_free(val);
	return 0;
}

static int
rpcServerCalls(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;
	rpc_func_t *f;
	register int i = 0;
	int len;
	char *val, str[MAXPATHLEN];

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	SLIST_FOREACH(f, &srv->srv_funcs, func_next)
		i++;
	len = i * STRSIZ;

	if (!(val = e_malloc(len))) {
		LOGERR;
		return -1;
	} else
		memset(val, 0, len);

	SLIST_FOREACH(f, &srv->srv_funcs, func_next)
		if (AIT_ADDR(&f->func_name)) {
			memset(str, 0, sizeof str);
			snprintf(str, sizeof str, "/%hu/0x%p; ", AIT_KEY(&f->func_name), 
					AIT_ADDR(&f->func_name));
			strlcat(val, str, len);
		}

	/* return variables */
	AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), val);
	e_free(val);
	return 0;
}

static int
rpcServerSessions(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	AIT_SET_I32(ait_getVars(&RPC_RETVARS(cli), 2), array_Size(srv->srv_clients));
	AIT_SET_U8(ait_getVars(&RPC_RETVARS(cli), 0), srv->srv_session.sess_version);
	AIT_SET_U8(ait_getVars(&RPC_RETVARS(cli), 1), srv->srv_session.sess_instance);

	return 0;
}

static int
rpcServerShutdown(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);
	
	srv->srv_blob.kill = 1;
	if (srv->srv_blob.tid)
		pthread_kill(srv->srv_blob.tid, SIGFBLOB);
	srv->srv_kill = 1;
	return 0;
}

static int
rpcServerPing(RPC_CALL_STDARGS)
{
	if (ntohl((u_long) rpc->call_req.flags) == RPC_REPLY)
		AIT_SET_U16(ait_getVars(&RPC_RETVARS(cli), 0), random() % USHRT_MAX);
	return 0;
}

/* ---------------------------------------------------- */

static int
rpcBLOBServerShutdown(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	rpc_freeBLOBCli(cli);

	srv->srv_blob.kill = 1;
	return 0;
}

static int
rpcBLOBServerVars(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;
	rpc_blob_t *b;
	register int i = 0;
	char *val, str[64];
	int len;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	if (srv->srv_blob.kill) {
		AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), "BLOB Server is killed");
		return 1;
	}

	TAILQ_FOREACH(b, &srv->srv_blob.blobs, blob_node)
		i++;
	len = i * sizeof str;

	if (!len) {
		AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), "");
		return 0;
	}

	if (!(val = e_malloc(len))) {
		LOGERR;
		return -1;
	} else
		memset(val, 0, len);

	TAILQ_FOREACH(b, &srv->srv_blob.blobs, blob_node) {
		memset(str, 0, sizeof str);
		snprintf(str, sizeof str, "0x%0X(%lu)=%p ", b->blob_var, (u_long) b->blob_len, b->blob_data);
		strlcat(val, str, len);
	}

	AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), val);
	e_free(val);
	return 0;
}

static int
rpcBLOBServerClients(RPC_CALL_STDARGS)
{
	rpc_srv_t *srv;
	rpc_cli_t *c;
	register int i;
	int len;
	const char *str = NULL;
	char *val;
	ait_val_t v;

	RPC_CALLBACK_CHECK_INPUT(cli);
	srv = RPC_SRV_SERVER(cli);

	if (srv->srv_blob.kill) {
		AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), "BLOB Server is killed");
		return 1;
	}

	len = array_Size(srv->srv_blob.clients) * STRSIZ;
	if (!(val = e_malloc(len))) {
		LOGERR;
		return -1;
	} else
		memset(val, 0, len);

	for (i = 0; i < array_Size(srv->srv_clients); i++) {
		c = array(srv->srv_blob.clients, i, rpc_cli_t*);
		if (!c)
			continue;

		str = e_n2addr(&c->cli_sa, &v);
		if (str)
			strlcat(val, (char*) str, len);
		else
			strlcat(val, "0.0.0.0", len);
		strlcat(val, " ", len);
		AIT_FREE_VAL(&v);
	}

	AIT_SET_STR(ait_getVars(&RPC_RETVARS(cli), 0), val);
	e_free(val);
	return 0;
}

/* ----------------------------------------------------------------- */

/*
 * rpc_register_srvPing() - Register ping service function
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int
rpc_register_srvPing(rpc_srv_t * __restrict srv)
{
	if (!srv)
		return -1;

	if (rpc_srv_registerCall(srv, CALL_SRVPING, rpcServerPing) < 1)
		return -1;

	return 0;
}

/*
 * rpc_register_srvServices() - Register internal service functions
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int
rpc_register_srvServices(rpc_srv_t * __restrict srv)
{
	if (!srv)
		return -1;

	if (rpc_srv_registerCall(srv, CALL_SRVSHUTDOWN, rpcServerShutdown) < 1)
		return -1;
	if (rpc_srv_registerCall(srv, CALL_SRVCLIENTS, rpcServerClients) < 1)
		return -1;
	if (rpc_srv_registerCall(srv, CALL_SRVSESSIONS, rpcServerSessions) < 1)
		return -1;
	if (rpc_srv_registerCall(srv, CALL_SRVCALLS, rpcServerCalls) < 1)
		return -1;

	return 0;
}

/*
 * rpc_register_blobServices() - Register internal service functions
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int
rpc_register_blobServices(rpc_srv_t * __restrict srv)
{
	if (!srv)
		return -1;

	if (rpc_srv_registerCall(srv, CALL_BLOBSHUTDOWN, rpcBLOBServerShutdown) < 1)
		return -1;
	if (rpc_srv_registerCall(srv, CALL_BLOBCLIENTS, rpcBLOBServerClients) < 1)
		return -1;
	if (rpc_srv_registerCall(srv, CALL_BLOBVARS, rpcBLOBServerVars) < 1)
		return -1;

	return 0;
}
